@@ -1,5 +1,4 @@
-/*
- * Ari Moisio <armoi@sci.fi> Aug 29 2000, based on skeleton effect
+/* Ari Moisio <armoi@sci.fi> Aug 29 2000, based on skeleton effect
  * Written by Chris Bagwell (cbagwell@sprynet.com) - March 16, 1999
  *
  * Copyright 1999 Chris Bagwell And Sundry Contributors
@@ -9,6 +8,8 @@
  * the consequences of using this software.
  */
 
+#include "sox_i.h"
+
 /* Fade curves */
 #define FADE_QUARTER    'q'     /* Quarter of sine wave, 0 to pi/2 */
 #define FADE_HALF       'h'     /* Half of sine wave, pi/2 to 1.5 * pi
@@ -19,19 +20,16 @@
 #define FADE_TRI        't'     /* Linear slope. */
 #define FADE_PAR        'p'     /* Inverted parabola. */
 
-#include <math.h>
 #include <string.h>
-#include "sox_i.h"
 
 /* Private data for fade file */
-typedef struct fadestuff
-{ /* These are measured as samples */
+typedef struct { /* These are measured as samples */
     sox_size_t in_start, in_stop, out_start, out_stop, samplesdone;
     char *in_stop_str, *out_start_str, *out_stop_str;
     char in_fadetype, out_fadetype;
     char do_out;
     int endpadwarned;
-} *fade_t;
+} priv_t;
 
 /* prototypes */
 static double fade_gain(sox_size_t index, sox_size_t range, int fadetype);
@@ -46,12 +44,12 @@ static double fade_gain(sox_size_t index, sox_size_t range, int fadetype);
 static int sox_fade_getopts(sox_effect_t * effp, int n, char **argv)
 {
 
-    fade_t fade = (fade_t) effp->priv;
+    priv_t * fade = (priv_t *) effp->priv;
     char t_char[2];
     int t_argno;
 
     if (n < 1 || n > 4)
-         return sox_usage(effp);
+         return lsx_usage(effp);
 
     /* because sample rate is unavailable at this point we store the
      * string off for later computations.
@@ -72,11 +70,11 @@ static int sox_fade_getopts(sox_effect_t * effp, int n, char **argv)
         fade->out_fadetype = 'l';
     }
 
-    fade->in_stop_str = (char *)xmalloc(strlen(argv[0])+1);
+    fade->in_stop_str = lsx_malloc(strlen(argv[0])+1);
     strcpy(fade->in_stop_str,argv[0]);
     /* Do a dummy parse to see if it will fail */
-    if (sox_parsesamples(0, fade->in_stop_str, &fade->in_stop, 't') == NULL)
-      return sox_usage(effp);
+    if (lsx_parsesamples(0., fade->in_stop_str, &fade->in_stop, 't') == NULL)
+      return lsx_usage(effp);
 
     fade->out_start_str = fade->out_stop_str = 0;
 
@@ -85,23 +83,23 @@ static int sox_fade_getopts(sox_effect_t * effp, int n, char **argv)
         /* See if there is fade-in/fade-out times/curves specified. */
         if(t_argno == 1)
         {
-            fade->out_stop_str = (char *)xmalloc(strlen(argv[t_argno])+1);
+            fade->out_stop_str = lsx_malloc(strlen(argv[t_argno])+1);
             strcpy(fade->out_stop_str,argv[t_argno]);
 
             /* Do a dummy parse to see if it will fail */
-            if (sox_parsesamples(0, fade->out_stop_str, 
+            if (lsx_parsesamples(0., fade->out_stop_str,
                                 &fade->out_stop, 't') == NULL)
-              return sox_usage(effp);
+              return lsx_usage(effp);
         }
         else
         {
-            fade->out_start_str = (char *)xmalloc(strlen(argv[t_argno])+1);
+            fade->out_start_str = lsx_malloc(strlen(argv[t_argno])+1);
             strcpy(fade->out_start_str,argv[t_argno]);
 
             /* Do a dummy parse to see if it will fail */
-            if (sox_parsesamples(0, fade->out_start_str, 
+            if (lsx_parsesamples(0., fade->out_start_str,
                                 &fade->out_start, 't') == NULL)
-              return sox_usage(effp);
+              return lsx_usage(effp);
         }
     } /* End for(t_argno) */
 
@@ -114,29 +112,37 @@ static int sox_fade_getopts(sox_effect_t * effp, int n, char **argv)
  */
 static int sox_fade_start(sox_effect_t * effp)
 {
-    fade_t fade = (fade_t) effp->priv;
+    priv_t * fade = (priv_t *) effp->priv;
 
     /* converting time values to samples */
     fade->in_start = 0;
-    if (sox_parsesamples(effp->ininfo.rate, fade->in_stop_str,
+    if (lsx_parsesamples(effp->in_signal.rate, fade->in_stop_str,
                         &fade->in_stop, 't') == NULL)
-      return sox_usage(effp);
+      return lsx_usage(effp);
 
     fade->do_out = 0;
     /* See if user specified a stop time */
     if (fade->out_stop_str)
     {
         fade->do_out = 1;
-        if (sox_parsesamples(effp->ininfo.rate, fade->out_stop_str,
+        if (lsx_parsesamples(effp->in_signal.rate, fade->out_stop_str,
                             &fade->out_stop, 't') == NULL)
-          return sox_usage(effp);
+          return lsx_usage(effp);
+
+        if (!fade->out_stop) {
+          fade->out_stop = effp->in_signal.length / effp->in_signal.channels;
+          if (!fade->out_stop) {
+            sox_fail("cannot fade out: audio length is neither known nor given");
+            return SOX_EOF;
+          }
+        }
 
         /* See if user wants to fade out. */
         if (fade->out_start_str)
         {
-            if (sox_parsesamples(effp->ininfo.rate, fade->out_start_str,
+            if (lsx_parsesamples(effp->in_signal.rate, fade->out_start_str,
                         &fade->out_start, 't') == NULL)
-              return sox_usage(effp);
+              return lsx_usage(effp);
             /* Fade time is relative to stop time. */
             fade->out_start = fade->out_stop - fade->out_start;
 
@@ -149,7 +155,7 @@ static int sox_fade_start(sox_effect_t * effp)
             fade->out_start = fade->out_stop - fade->in_stop;
     }
     else
-        /* If not specified then user wants to process all 
+        /* If not specified then user wants to process all
          * of file.  Use a value of zero to indicate this.
          */
         fade->out_stop = 0;
@@ -176,10 +182,10 @@ static int sox_fade_start(sox_effect_t * effp)
  * Processed signed long samples from ibuf to obuf.
  * Return number of samples processed.
  */
-static int sox_fade_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obuf, 
+static int sox_fade_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obuf,
                  sox_size_t *isamp, sox_size_t *osamp)
 {
-    fade_t fade = (fade_t) effp->priv;
+    priv_t * fade = (priv_t *) effp->priv;
     /* len is total samples, chcnt counts channels */
     int len = 0, t_output = 1, more_output = 1;
     sox_sample_t t_ibuf;
@@ -238,7 +244,7 @@ static int sox_fade_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_samp
 
         /* Process next channel */
         chcnt++;
-        if (chcnt >= effp->ininfo.channels)
+        if (chcnt >= effp->in_signal.channels)
         { /* all channels of this sample processed */
             chcnt = 0;
             fade->samplesdone += 1;
@@ -259,7 +265,7 @@ static int sox_fade_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_samp
  */
 static int sox_fade_drain(sox_effect_t * effp, sox_sample_t *obuf, sox_size_t *osamp)
 {
-    fade_t fade = (fade_t) effp->priv;
+    priv_t * fade = (priv_t *) effp->priv;
     int len;
     sox_size_t t_chan = 0;
 
@@ -281,7 +287,7 @@ static int sox_fade_drain(sox_effect_t * effp, sox_sample_t *obuf, sox_size_t *o
         *osamp += 1;
 
         t_chan++;
-        if (t_chan >= effp->ininfo.channels)
+        if (t_chan >= effp->in_signal.channels)
         {
             fade->samplesdone += 1;
             t_chan = 0;
@@ -300,7 +306,7 @@ static int sox_fade_drain(sox_effect_t * effp, sox_sample_t *obuf, sox_size_t *o
  */
 static int kill(sox_effect_t * effp)
 {
-    fade_t fade = (fade_t) effp->priv;
+    priv_t * fade = (priv_t *) effp->priv;
 
     free(fade->in_stop_str);
     free(fade->out_start_str);
@@ -361,7 +367,7 @@ static sox_effect_handler_t sox_fade_effect = {
   sox_fade_flow,
   sox_fade_drain,
   NULL,
-  kill
+  kill, sizeof(priv_t)
 };
 
 const sox_effect_handler_t *sox_fade_effect_fn(void)
