@@ -1,33 +1,32 @@
-/*
- * libSoX lpc-10 format.
+/* libSoX lpc-10 format.
  *
  * Copyright 2007 Reuben Thomas <rrt@sc3d.org>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or (at
+ * your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, write to the Free Software
- * Foundation, Fifth Floor, 51 Franklin Street, Boston, MA 02111-1301,
- * USA.  */
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 
 #include "sox_i.h"
 #include "../lpc10/lpc10.h"
 
 /* Private data */
-typedef struct lpcpriv {
+typedef struct {
   struct lpc10_encoder_state *encst;
   float speech[LPC10_SAMPLES_PER_FRAME];
   unsigned samples;
   struct lpc10_decoder_state *decst;
-} *lpcpriv_t;
+} priv_t;
 
 /*
   Write the bits in bits[0] through bits[len-1] to file f, in "packed"
@@ -71,7 +70,7 @@ static void write_bits(sox_format_t * ft, INT32 *bits, int len)
      * then mask won't yet be 0.  */
     mask >>= 1;
     if ((mask == 0) || (i == len-1)) {
-      sox_writeb(ft, data);
+      lsx_writeb(ft, data);
       data = 0;
       mask = 0x80;
     }
@@ -104,8 +103,8 @@ static int read_bits(sox_format_t * ft, INT32 *bits, int len)
   /* Unpack the array bits into coded_frame. */
   for (i = 0; i < len; i++) {
     if (i % 8 == 0) {
-      sox_readb(ft, &c);
-      if (sox_eof(ft)) {
+      lsx_read_b_buf(ft, &c, 1);
+      if (lsx_eof(ft)) {
         return (i);
       }
     }
@@ -120,22 +119,19 @@ static int read_bits(sox_format_t * ft, INT32 *bits, int len)
 
 static int startread(sox_format_t * ft)
 {
-  lpcpriv_t lpc = (lpcpriv_t)ft->priv;
+  priv_t * lpc = (priv_t *)ft->priv;
 
   if ((lpc->decst = create_lpc10_decoder_state()) == NULL) {
     fprintf(stderr, "lpc10 could not allocate decoder state");
     return SOX_EOF;
   }
   lpc->samples = LPC10_SAMPLES_PER_FRAME;
-  ft->signal.size = SOX_SIZE_16BIT;
-  ft->signal.encoding = SOX_ENCODING_SIGN2;
-
-  return SOX_SUCCESS;
+  return lsx_check_read_params(ft, 1, 8000., SOX_ENCODING_LPC10, 0, (off_t)0);
 }
 
-static int startwrite(sox_format_t * ft) 
+static int startwrite(sox_format_t * ft)
 {
-  lpcpriv_t lpc = (lpcpriv_t)ft->priv;
+  priv_t * lpc = (priv_t *)ft->priv;
 
   if ((lpc->encst = create_lpc10_encoder_state()) == NULL) {
     fprintf(stderr, "lpc10 could not allocate encoder state");
@@ -146,9 +142,9 @@ static int startwrite(sox_format_t * ft)
   return SOX_SUCCESS;
 }
 
-static sox_size_t read(sox_format_t * ft, sox_sample_t *buf, sox_size_t len)
+static sox_size_t read_samples(sox_format_t * ft, sox_sample_t *buf, sox_size_t len)
 {
-  lpcpriv_t lpc = (lpcpriv_t)ft->priv;
+  priv_t * lpc = (priv_t *)ft->priv;
   sox_size_t nread = 0;
 
   while (nread < len) {
@@ -170,9 +166,9 @@ static sox_size_t read(sox_format_t * ft, sox_sample_t *buf, sox_size_t len)
   return nread;
 }
 
-static sox_size_t write(sox_format_t * ft, const sox_sample_t *buf, sox_size_t len)
+static sox_size_t write_samples(sox_format_t * ft, const sox_sample_t *buf, sox_size_t len)
 {
-  lpcpriv_t lpc = (lpcpriv_t)ft->priv;
+  priv_t * lpc = (priv_t *)ft->priv;
   sox_size_t nwritten = 0;
 
   while (len > 0) {
@@ -195,7 +191,7 @@ static sox_size_t write(sox_format_t * ft, const sox_sample_t *buf, sox_size_t l
 
 static int stopread(sox_format_t * ft)
 {
-  lpcpriv_t lpc = (lpcpriv_t)ft->priv;
+  priv_t * lpc = (priv_t *)ft->priv;
 
   free(lpc->decst);
 
@@ -204,35 +200,23 @@ static int stopread(sox_format_t * ft)
 
 static int stopwrite(sox_format_t * ft)
 {
-  lpcpriv_t lpc = (lpcpriv_t)ft->priv;
+  priv_t * lpc = (priv_t *)ft->priv;
 
   free(lpc->encst);
 
   return SOX_SUCCESS;
 }
 
-/* LPC-10 */
-static const char *lpc10names[] = {
-  "lpc",
-  "lpc10",
-  NULL
-};
-
-static sox_format_handler_t sox_lpc10_format = {
-  lpc10names,
-  0,
-  startread,
-  read,
-  stopread,
-  startwrite,
-  write,
-  stopwrite,
-  sox_format_nothing_seek
-};
-
-const sox_format_handler_t *sox_lpc10_format_fn(void);
-
-const sox_format_handler_t *sox_lpc10_format_fn(void)
+SOX_FORMAT_HANDLER(lpc10)
 {
-  return &sox_lpc10_format;
+  static char const * const names[] = {"lpc10", "lpc", NULL};
+  static sox_rate_t   const write_rates[] = {8000, 0};
+  static unsigned     const write_encodings[] = {SOX_ENCODING_LPC10, 0, 0};
+  static sox_format_handler_t handler = {SOX_LIB_VERSION_CODE,
+    "Low bandwidth, robotic sounding speech compression", names, SOX_FILE_MONO,
+    startread, read_samples, stopread,
+    startwrite, write_samples, stopwrite,
+    NULL, write_encodings, write_rates, sizeof(priv_t)
+  };
+  return &handler;
 }
